library feedify;

import 'dart:io' show Platform;

import 'package:feedify/const.dart';
import 'package:feedify/logs.dart';
import 'package:feedify/src/manager/feed_api_manager.dart';
import 'package:feedify/src/manager/notification_manager.dart';
import 'package:feedify/src/model/model_device_app.dart';
import 'package:feedify/src/model/model_firebase_app.dart';
import 'package:feedify/src/model/notification/model_notification.dart';
import 'package:feedify/src/utils/notification_util.dart';
import 'package:firebase_core/firebase_core.dart';
import 'package:firebase_messaging/firebase_messaging.dart';

import 'src/storage/app-prefs.dart';

/// entry point for SDK, this class contains all the configuration related methods that needs
/// to be used at runtime
Future<void> _firebaseMessagingBackgroundHandler(RemoteMessage message) async {
  Logs.d('background onMessage received');
  _handleNotification(message);
}

class FeedSDK {
  static String feedUser = '';
  static String feedKey = '';
  static String feedDomain = '';
  static String notificationIco = '';
  static String appName = '';
  static final FeedSDK _singleton = FeedSDK._internal();
  AppPreferences? _appPreferences;

  FeedSDK._internal();

  _FeedSDK() {}

  /// used to initialise the SDK and Firebase
  ///
  /// [feedifyUser] username provided from Feedify console,
  /// [feedKey] key provided from Feedify console and
  /// [feedDomain] domain provided from Feedify console
  /// all the paramters are mandatory, if not provided then [UnimplementedError] is thrown
  static FeedSDK getInstance(
      {required feedifyUser, required feedifyKey, required feedifyDomain}) {
    if (feedifyUser.isEmpty || feedifyDomain.isEmpty || feedifyKey.isEmpty) {
      throw UnimplementedError('kindly provide valid values');
    }
    feedKey = feedifyKey;
    feedDomain = feedifyDomain;
    feedUser = feedifyUser;
    return _singleton;
  }

  /// icon to be used on the notification, make sure notification icon is available for all screen sizes or else notification will fail to show!
  ///
  ///  [notificationIcon] - drawable icon to be used as icon to be shown inside notifications
  /// like: 'resource://drawable/ic_notification'
  static void setNotificationIcon(String notificationIcon) {
    notificationIco = notificationIcon;
  }

  static void getUUID() {
    var mobileDevice = await ModelDeviceApp.getInstance();
    if (mobileDevice != null) {
      return mobileDevice.getUUID();
    }
  }

  ///used to initialise the SDK and Firebase
  ///- Note: All the required paramters can be found in google-services.json file downloaded from Firebase console
  ///except [appName] which can be any String
  /// [notificationDataCallback] all notification payload data will be passed to the calling app if this is not null

  Future<void> init({
    required String appName,
    required String projectId,
    required String firebaseCurrentApiKey,
    required String firebaseMobileSdkAppId,
    required String firebaseStorageBucket,
    String? firebaseUrl,
    required String firebaseProjectNumber,
    required Function(Map<String, String?>)? notificationDataCallback,
  }) async {
    Logs.d('started init');
    await _initialiseApp(_getFirebaseModel(
      appName,
      projectId,
      firebaseCurrentApiKey,
      firebaseMobileSdkAppId,
      firebaseStorageBucket,
      firebaseUrl,
      firebaseProjectNumber,
    ));
    FeedSDK.appName = appName;
    Logs.d('ended init');

    if (Platform.isIOS) {
      _requestPermissionForIos();
    }
    FeedApiManager.getInstance().register(_appPreferences);
    _initialiseNotifications(notificationDataCallback);
    _addMessageListener();
  }

  _initialiseApp(ModelFirebaseApp modelFirebaseApp) async {
    await Firebase.initializeApp(
            name: modelFirebaseApp.appName,
            options: FirebaseOptions(
                projectId: modelFirebaseApp.projectId,
                apiKey: modelFirebaseApp.firebaseCurrentApiKey,
                appId: modelFirebaseApp.firebaseMobileSdkAppId,
                databaseURL: modelFirebaseApp.firebaseUrl,
                messagingSenderId: modelFirebaseApp.firebaseProjectNumber,
                storageBucket: modelFirebaseApp.firebaseStorageBucket))
        .whenComplete(() async {
      Logs.d('Firebase Initialised...successfully');
      var secondaryApp = Firebase.app(modelFirebaseApp.appName);
      Logs.d('App name :: $secondaryApp');
      await _fetchToken();
    });
  }

  _fetchToken() async {
    Logs.d('getToken called...');
    String? token = await FirebaseMessaging.instance.getToken();
    Logs.d('token :: $token');
    bool? saved = await saveToken(token);
    Logs.d(saved == true ? 'token saved' : 'token failed to save');
  }

  /**
   * _requestPermissionForIos is requried for Firebase
   * Permission is compulsory
   */

  _requestPermissionForIos() async {
    FirebaseMessaging messaging = FirebaseMessaging.instance;
    NotificationSettings settings = await messaging.requestPermission(
      alert: true,
      announcement: false,
      badge: true,
      carPlay: false,
      criticalAlert: false,
      provisional: false,
      sound: true,
    );
  }

  ///provides Firebase device token unique for every device from preferences
  Future<String?> getToken() async {
    _appPreferences ??= AppPreferences();
    return await _appPreferences?.getString(AppPreferences.KEY_FCM);
  }

  ///saves Firebase device token [fcmToken] unique for every device

  Future<bool?> saveToken(String? fcmToken) async {
    _appPreferences ??= AppPreferences();
    return await _appPreferences?.saveString(AppPreferences.KEY_FCM, fcmToken);
  }

  ModelFirebaseApp _getFirebaseModel(
      String appName,
      String projectId,
      String firebaseCurrentApiKey,
      String firebaseMobileSdkAppId,
      String firebaseStorageBucket,
      String? firebaseUrl,
      String firebaseProjectNumber) {
    return ModelFirebaseApp(
        appName,
        projectId,
        firebaseCurrentApiKey,
        firebaseMobileSdkAppId,
        firebaseStorageBucket,
        firebaseUrl,
        firebaseProjectNumber);
  }

  void _addMessageListener() async {
    await FirebaseMessaging.instance
        .setForegroundNotificationPresentationOptions(
      alert: true,
      badge: true,
      sound: true,
    );

    FirebaseMessaging.instance.onTokenRefresh.listen((event) {
      saveToken(event);
    });

    FirebaseMessaging.onMessage.listen((event) {
      Logs.d('onMessage received');
      _handleNotification(event);
    });

    FirebaseMessaging.onMessageOpenedApp.listen((event) {
      Logs.d('onMessageOpenedApp');
    });

    FirebaseMessaging.onBackgroundMessage(_firebaseMessagingBackgroundHandler);
  }

  void _initialiseNotifications(
      Function(Map<String, String?>)? notificationDataCallback) {
    NotificationManager.getInstance().init(notificationDataCallback);
  }

  ///provides notification data received from feedify server, when you are done with the data, make sure to clear it by calling [clearNotificationData]
  /// NOT WORKING - WIP
  static Map<String, String> _fetchNotificationData() {
    return NotificationUtils().getSavedNotificationData();
  }

  /// clear push notification message data
  /// NOT WORKING - WIP
  static void _clearNotificationData() {
    NotificationUtils.getInstance().clearNotificationData();
  }

  /// Retrieve notification payloads data object
  static Future<ModelNotification> retrieveNotificationPayloadsData() async {
    final String? notificationData = await FeedSDK._singleton._appPreferences
        ?.getString(Constants.notificationId);
    return ModelNotification.fromJson(notificationData);
  }
}

ModelNotification _getModelNotification(RemoteMessage event) {
  return ModelNotification.fromJson(event.data);
}

void _handleNotification(RemoteMessage event) {
  var notificationData = _getModelNotification(event);
  Logs.d('notification model :: ${notificationData.toJson()}');
  if (notificationData.type == 'ad') {
    Logs.d('fetchAd  started ...');
    _fetchAd(event.data);
  } else {
    // var notificationData = _getModelNotification(event);
    _showNotification(notificationData);
    _saveNotificationData(notificationData);
  }
}

void _saveNotificationData(ModelNotification notificationData) async {
  await AppPreferences().savePayloadsInPreference(notificationData);
}

void _showNotification(ModelNotification? notificationData) {
  if (notificationData != null) {
    NotificationManager.getInstance().showNotification(notificationData);
    _callDeliveryCountApi(notificationData);
  }
}

void _callDeliveryCountApi(ModelNotification notificationData) {
  String? id = notificationData.id;
  String? type = notificationData.type;
  if (id != null && id.isNotEmpty && type != null && type.isNotEmpty) {
    FeedApiManager.getInstance()
        .deliveryCount(notificationData.id!, notificationData.type!);
  }
}

void _fetchAd(Map<String, dynamic> notificationData) async {
  var response = await FeedApiManager.getInstance().fetchAd(notificationData);
  Logs.d('fetchAd model :: ${response?.id}');
  _showNotification(response);
  Logs.d(response.toString());
}
